# coding: utf-8

# flake8: noqa
"""
    BeeL Public API

    Public invoicing API for self-employed professionals in Spain with full VeriFactu support.  **Authentication:** This API uses **API Keys** for authentication. Endpoints are designed for external integrations, automation, and programmatic access.  For browser-based authentication, use the Private API (with Session Cookies).  **Key Features:** - VeriFactu regulatory compliance - Complete invoice management (standard, corrective, simplified) - Customer management - Tax and invoice series configuration - PDF generation - Email sending - Duplicate prevention (idempotency)  **Idempotency:** Creation endpoints (POST) support the `Idempotency-Key` header to prevent duplicates in case of retries.  **Identifiers:** - All resources use UUIDs for technical identification - Invoice numbers are human-readable and searchable 

    The version of the OpenAPI document: 1.0.0
    Contact: it@beel.es
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


# import models into model package
from beel_generated.models.address import Address
from beel_generated.models.alternative_identifier import AlternativeIdentifier
from beel_generated.models.change_invoices_status_bulk200_response import ChangeInvoicesStatusBulk200Response
from beel_generated.models.change_invoices_status_bulk200_response_all_of_data import ChangeInvoicesStatusBulk200ResponseAllOfData
from beel_generated.models.change_invoices_status_bulk200_response_all_of_data_failures_inner import ChangeInvoicesStatusBulk200ResponseAllOfDataFailuresInner
from beel_generated.models.change_invoices_status_bulk_request import ChangeInvoicesStatusBulkRequest
from beel_generated.models.color_option import ColorOption
from beel_generated.models.company_api_key_data import CompanyApiKeyData
from beel_generated.models.company_data import CompanyData
from beel_generated.models.company_response201 import CompanyResponse201
from beel_generated.models.counter_reset import CounterReset
from beel_generated.models.create_company_api_key201_response import CreateCompanyApiKey201Response
from beel_generated.models.create_company_api_key201_response_data import CreateCompanyApiKey201ResponseData
from beel_generated.models.create_company_api_key_request import CreateCompanyApiKeyRequest
from beel_generated.models.create_company_request import CreateCompanyRequest
from beel_generated.models.create_corrective_invoice_request import CreateCorrectiveInvoiceRequest
from beel_generated.models.create_corrective_invoice_request_lines_inner import CreateCorrectiveInvoiceRequestLinesInner
from beel_generated.models.create_customer201_response import CreateCustomer201Response
from beel_generated.models.create_customer_request import CreateCustomerRequest
from beel_generated.models.create_customers_bulk200_response import CreateCustomersBulk200Response
from beel_generated.models.create_customers_bulk201_response import CreateCustomersBulk201Response
from beel_generated.models.create_customers_bulk201_response_all_of_data import CreateCustomersBulk201ResponseAllOfData
from beel_generated.models.create_customers_bulk422_response import CreateCustomersBulk422Response
from beel_generated.models.create_customers_bulk422_response_all_of_error import CreateCustomersBulk422ResponseAllOfError
from beel_generated.models.create_customers_bulk422_response_all_of_error_errors_inner import CreateCustomersBulk422ResponseAllOfErrorErrorsInner
from beel_generated.models.create_customers_bulk_request import CreateCustomersBulkRequest
from beel_generated.models.create_invoice201_response import CreateInvoice201Response
from beel_generated.models.create_invoice400_response import CreateInvoice400Response
from beel_generated.models.create_invoice400_response_all_of_error import CreateInvoice400ResponseAllOfError
from beel_generated.models.create_invoice_request import CreateInvoiceRequest
from beel_generated.models.create_invoice_request_lines_inner import CreateInvoiceRequestLinesInner
from beel_generated.models.create_invoices_bulk201_response import CreateInvoicesBulk201Response
from beel_generated.models.create_invoices_bulk201_response_all_of_data import CreateInvoicesBulk201ResponseAllOfData
from beel_generated.models.create_invoices_bulk422_response import CreateInvoicesBulk422Response
from beel_generated.models.create_invoices_bulk422_response_all_of_error import CreateInvoicesBulk422ResponseAllOfError
from beel_generated.models.create_invoices_bulk422_response_all_of_error_errors_inner import CreateInvoicesBulk422ResponseAllOfErrorErrorsInner
from beel_generated.models.create_invoices_bulk_request import CreateInvoicesBulkRequest
from beel_generated.models.create_invoices_bulk_request_invoices_inner import CreateInvoicesBulkRequestInvoicesInner
from beel_generated.models.create_product201_response import CreateProduct201Response
from beel_generated.models.create_product_request import CreateProductRequest
from beel_generated.models.create_products_bulk200_response import CreateProductsBulk200Response
from beel_generated.models.create_products_bulk200_response_all_of_data import CreateProductsBulk200ResponseAllOfData
from beel_generated.models.create_products_bulk200_response_all_of_data_errors_inner import CreateProductsBulk200ResponseAllOfDataErrorsInner
from beel_generated.models.create_products_bulk200_response_all_of_data_summary import CreateProductsBulk200ResponseAllOfDataSummary
from beel_generated.models.create_products_bulk_request import CreateProductsBulkRequest
from beel_generated.models.create_series201_response import CreateSeries201Response
from beel_generated.models.create_series_request import CreateSeriesRequest
from beel_generated.models.create_webhook_subscription201_response import CreateWebhookSubscription201Response
from beel_generated.models.create_webhook_subscription_request import CreateWebhookSubscriptionRequest
from beel_generated.models.csv_customer_preview import CsvCustomerPreview
from beel_generated.models.csv_import_metadata import CsvImportMetadata
from beel_generated.models.csv_import_preview_result import CsvImportPreviewResult
from beel_generated.models.csv_import_result import CsvImportResult
from beel_generated.models.csv_import_statistics import CsvImportStatistics
from beel_generated.models.csv_validation_error import CsvValidationError
from beel_generated.models.customer import Customer
from beel_generated.models.customer_validation_error import CustomerValidationError
from beel_generated.models.customer_validation_item import CustomerValidationItem
from beel_generated.models.customer_validation_metadata import CustomerValidationMetadata
from beel_generated.models.customer_validation_statistics import CustomerValidationStatistics
from beel_generated.models.customer_validation_unified_result import CustomerValidationUnifiedResult
from beel_generated.models.customer_validation_warning import CustomerValidationWarning
from beel_generated.models.deactivate_customer200_response import DeactivateCustomer200Response
from beel_generated.models.deactivate_customer200_response_all_of_data import DeactivateCustomer200ResponseAllOfData
from beel_generated.models.deactivate_customers_bulk200_response import DeactivateCustomersBulk200Response
from beel_generated.models.deactivate_customers_bulk200_response_all_of_data import DeactivateCustomersBulk200ResponseAllOfData
from beel_generated.models.deactivate_customers_bulk200_response_all_of_data_errors_inner import DeactivateCustomersBulk200ResponseAllOfDataErrorsInner
from beel_generated.models.deactivate_customers_bulk200_response_all_of_data_errors_inner_error import DeactivateCustomersBulk200ResponseAllOfDataErrorsInnerError
from beel_generated.models.delete_products_bulk200_response import DeleteProductsBulk200Response
from beel_generated.models.delete_products_bulk200_response_all_of_data import DeleteProductsBulk200ResponseAllOfData
from beel_generated.models.delete_products_bulk200_response_all_of_data_errors_inner import DeleteProductsBulk200ResponseAllOfDataErrorsInner
from beel_generated.models.delete_products_bulk_request import DeleteProductsBulkRequest
from beel_generated.models.download_invoices_pdf_bulk_request import DownloadInvoicesPdfBulkRequest
from beel_generated.models.duplicate_invoice_request import DuplicateInvoiceRequest
from beel_generated.models.email_configuration import EmailConfiguration
from beel_generated.models.entity_type import EntityType
from beel_generated.models.equivalence_surcharge import EquivalenceSurcharge
from beel_generated.models.equivalence_surcharge_percentage import EquivalenceSurchargePercentage
from beel_generated.models.error_detail import ErrorDetail
from beel_generated.models.error_response import ErrorResponse
from beel_generated.models.export_invoices_excel_request import ExportInvoicesExcelRequest
from beel_generated.models.field_deserialization_error import FieldDeserializationError
from beel_generated.models.generation_action import GenerationAction
from beel_generated.models.get_invoice_customization_options200_response import GetInvoiceCustomizationOptions200Response
from beel_generated.models.get_public_metrics200_response import GetPublicMetrics200Response
from beel_generated.models.get_tax_configuration200_response import GetTaxConfiguration200Response
from beel_generated.models.get_tax_types200_response import GetTaxTypes200Response
from beel_generated.models.get_tax_types200_response_all_of_data import GetTaxTypes200ResponseAllOfData
from beel_generated.models.get_veri_factu_configuration200_response import GetVeriFactuConfiguration200Response
from beel_generated.models.import_customers_csv_preview400_response import ImportCustomersCsvPreview400Response
from beel_generated.models.import_customers_csv_preview400_response_all_of_error import ImportCustomersCsvPreview400ResponseAllOfError
from beel_generated.models.import_customers_csv_preview400_response_all_of_error_details import ImportCustomersCsvPreview400ResponseAllOfErrorDetails
from beel_generated.models.invoice import Invoice
from beel_generated.models.invoice_attachments_inner import InvoiceAttachmentsInner
from beel_generated.models.invoice_customization_options_response import InvoiceCustomizationOptionsResponse
from beel_generated.models.invoice_line import InvoiceLine
from beel_generated.models.invoice_pdf_response import InvoicePdfResponse
from beel_generated.models.invoice_pdf_response_data import InvoicePdfResponseData
from beel_generated.models.invoice_processing_options import InvoiceProcessingOptions
from beel_generated.models.invoice_sending_history_inner import InvoiceSendingHistoryInner
from beel_generated.models.invoice_series import InvoiceSeries
from beel_generated.models.invoice_status import InvoiceStatus
from beel_generated.models.invoice_template_option import InvoiceTemplateOption
from beel_generated.models.invoice_totals import InvoiceTotals
from beel_generated.models.invoice_totals_surcharge_breakdown_inner import InvoiceTotalsSurchargeBreakdownInner
from beel_generated.models.invoice_totals_vat_breakdown_inner import InvoiceTotalsVatBreakdownInner
from beel_generated.models.invoice_type import InvoiceType
from beel_generated.models.irpf_category import IrpfCategory
from beel_generated.models.irpf_percentage import IrpfPercentage
from beel_generated.models.irpf_type import IrpfType
from beel_generated.models.issuer_data import IssuerData
from beel_generated.models.language import Language
from beel_generated.models.legal_representative import LegalRepresentative
from beel_generated.models.legal_representative_address import LegalRepresentativeAddress
from beel_generated.models.list_companies200_response import ListCompanies200Response
from beel_generated.models.list_company_api_keys200_response import ListCompanyApiKeys200Response
from beel_generated.models.list_customers200_response import ListCustomers200Response
from beel_generated.models.list_customers200_response_all_of_data import ListCustomers200ResponseAllOfData
from beel_generated.models.list_invoices200_response import ListInvoices200Response
from beel_generated.models.list_invoices200_response_all_of_data import ListInvoices200ResponseAllOfData
from beel_generated.models.list_products200_response import ListProducts200Response
from beel_generated.models.list_products200_response_all_of_data import ListProducts200ResponseAllOfData
from beel_generated.models.list_series200_response import ListSeries200Response
from beel_generated.models.list_webhook_deliveries200_response import ListWebhookDeliveries200Response
from beel_generated.models.list_webhook_subscriptions200_response import ListWebhookSubscriptions200Response
from beel_generated.models.list_webhook_subscriptions200_response_all_of_data import ListWebhookSubscriptions200ResponseAllOfData
from beel_generated.models.mark_invoice_paid_request import MarkInvoicePaidRequest
from beel_generated.models.mark_invoice_sent_request import MarkInvoiceSentRequest
from beel_generated.models.nif_validation_status import NifValidationStatus
from beel_generated.models.pagination import Pagination
from beel_generated.models.payment_info import PaymentInfo
from beel_generated.models.payment_method import PaymentMethod
from beel_generated.models.product import Product
from beel_generated.models.product_category import ProductCategory
from beel_generated.models.product_query_params import ProductQueryParams
from beel_generated.models.product_response import ProductResponse
from beel_generated.models.products_list_response import ProductsListResponse
from beel_generated.models.products_list_response_all_of_data import ProductsListResponseAllOfData
from beel_generated.models.public_metrics_response import PublicMetricsResponse
from beel_generated.models.recipient import Recipient
from beel_generated.models.recipient_alternative_id import RecipientAlternativeId
from beel_generated.models.recipient_data import RecipientData
from beel_generated.models.recipient_data_alternative_id import RecipientDataAlternativeId
from beel_generated.models.rectification_type import RectificationType
from beel_generated.models.regime_key import RegimeKey
from beel_generated.models.representation_action_response import RepresentationActionResponse
from beel_generated.models.representation_action_response_data import RepresentationActionResponseData
from beel_generated.models.representation_download_response import RepresentationDownloadResponse
from beel_generated.models.representation_download_response_data import RepresentationDownloadResponseData
from beel_generated.models.representation_status_response import RepresentationStatusResponse
from beel_generated.models.representation_status_response_data import RepresentationStatusResponseData
from beel_generated.models.reschedule_invoice_request import RescheduleInvoiceRequest
from beel_generated.models.response_meta import ResponseMeta
from beel_generated.models.retry_webhook_delivery200_response import RetryWebhookDelivery200Response
from beel_generated.models.schedule_invoice_request import ScheduleInvoiceRequest
from beel_generated.models.schemas_tax_regime import SchemasTaxRegime
from beel_generated.models.search_products200_response import SearchProducts200Response
from beel_generated.models.send_email_request import SendEmailRequest
from beel_generated.models.send_invoice_email200_response import SendInvoiceEmail200Response
from beel_generated.models.send_invoice_email200_response_all_of_data import SendInvoiceEmail200ResponseAllOfData
from beel_generated.models.send_invoices_bulk_email200_response import SendInvoicesBulkEmail200Response
from beel_generated.models.send_invoices_bulk_email200_response_all_of_data import SendInvoicesBulkEmail200ResponseAllOfData
from beel_generated.models.send_invoices_bulk_email200_response_all_of_data_failures_inner import SendInvoicesBulkEmail200ResponseAllOfDataFailuresInner
from beel_generated.models.send_invoices_bulk_email_request import SendInvoicesBulkEmailRequest
from beel_generated.models.series_info import SeriesInfo
from beel_generated.models.submit_invoice_veri_factu200_response import SubmitInvoiceVeriFactu200Response
from beel_generated.models.submit_invoice_veri_factu200_response_all_of_data import SubmitInvoiceVeriFactu200ResponseAllOfData
from beel_generated.models.success_response import SuccessResponse
from beel_generated.models.tax_configuration import TaxConfiguration
from beel_generated.models.tax_configuration_default_main_tax import TaxConfigurationDefaultMainTax
from beel_generated.models.tax_info import TaxInfo
from beel_generated.models.tax_percentage import TaxPercentage
from beel_generated.models.tax_regime import TaxRegime
from beel_generated.models.tax_type import TaxType
from beel_generated.models.template_csv_info import TemplateCsvInfo
from beel_generated.models.update_company_request import UpdateCompanyRequest
from beel_generated.models.update_customer_request import UpdateCustomerRequest
from beel_generated.models.update_invoice_request import UpdateInvoiceRequest
from beel_generated.models.update_invoice_request_lines_inner import UpdateInvoiceRequestLinesInner
from beel_generated.models.update_language200_response import UpdateLanguage200Response
from beel_generated.models.update_language200_response_all_of_data import UpdateLanguage200ResponseAllOfData
from beel_generated.models.update_language_request import UpdateLanguageRequest
from beel_generated.models.update_product_request import UpdateProductRequest
from beel_generated.models.update_series_request import UpdateSeriesRequest
from beel_generated.models.update_tax_configuration_request import UpdateTaxConfigurationRequest
from beel_generated.models.update_webhook_subscription200_response import UpdateWebhookSubscription200Response
from beel_generated.models.update_webhook_subscription_request import UpdateWebhookSubscriptionRequest
from beel_generated.models.validate_nif200_response import ValidateNif200Response
from beel_generated.models.validate_nif_request import ValidateNifRequest
from beel_generated.models.validate_nif_response import ValidateNifResponse
from beel_generated.models.validation_status import ValidationStatus
from beel_generated.models.vat_category import VatCategory
from beel_generated.models.vat_percentage import VatPercentage
from beel_generated.models.vat_type import VatType
from beel_generated.models.veri_factu import VeriFactu
from beel_generated.models.veri_factu_configuration import VeriFactuConfiguration
from beel_generated.models.veri_factu_rectification_code import VeriFactuRectificationCode
from beel_generated.models.veri_factu_regime_key import VeriFactuRegimeKey
from beel_generated.models.void_invoice_request import VoidInvoiceRequest
from beel_generated.models.webhook_delivery_log import WebhookDeliveryLog
from beel_generated.models.webhook_event import WebhookEvent
from beel_generated.models.webhook_event_data import WebhookEventData
from beel_generated.models.webhook_event_data_veri_factu_status_updated import WebhookEventDataVeriFactuStatusUpdated
from beel_generated.models.webhook_subscription import WebhookSubscription
from beel_generated.models.webhook_subscription_with_secret import WebhookSubscriptionWithSecret
